package cn.ibizlab.core.rule.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.rule.domain.ExecResult;
import cn.ibizlab.core.rule.filter.ExecResultSearchContext;
import cn.ibizlab.core.rule.service.IExecResultService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.rule.mapper.ExecResultMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[执行结果] 服务对象接口实现
 */
@Slf4j
@Service("ExecResultServiceImpl")
public class ExecResultServiceImpl extends ServiceImpl<ExecResultMapper, ExecResult> implements IExecResultService {

    @Autowired
    @Lazy
    protected cn.ibizlab.core.rule.service.IRuleItemService ruleitemService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(ExecResult et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getId()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<ExecResult> list) {
        list.forEach(item->fillParentData(item));
        this.saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(ExecResult et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("ru_execresultid", et.getId()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getId()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<ExecResult> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public ExecResult get(String key) {
        ExecResult et = getById(key);
        if(et == null){
            et = new ExecResult();
            et.setId(key);
        }
        else {
        }
        return et;
    }

    @Override
    public ExecResult getDraft(ExecResult et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(ExecResult et) {
        return (!ObjectUtils.isEmpty(et.getId())) && (!Objects.isNull(this.getById(et.getId())));
    }
    @Override
    @Transactional
    public boolean save(ExecResult et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(ExecResult et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<ExecResult> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<ExecResult> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<ExecResult> selectByRuleId(String rule_id) {
        return baseMapper.selectByRuleId(rule_id);
    }
    @Override
    public void removeByRuleId(String rule_id) {
        this.remove(new QueryWrapper<ExecResult>().eq("ruleid",rule_id));
    }


    /**
     * 查询集合 数据集
     */
    @Override
    public Page<ExecResult> searchDefault(ExecResultSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<ExecResult> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<ExecResult>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(ExecResult et){
        //实体关系[DER1N_RU_EXECRESULT_RU_ITEM_RULEID]
        if(!ObjectUtils.isEmpty(et.getRuleId())){
            cn.ibizlab.core.rule.domain.RuleItem rule=et.getRule();
            if(ObjectUtils.isEmpty(rule)){
                cn.ibizlab.core.rule.domain.RuleItem majorEntity=ruleitemService.get(et.getRuleId());
                et.setRule(majorEntity);
                rule=majorEntity;
            }
            et.setRuleName(rule.getRuleName());
            et.setSystemId(rule.getSystemId());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }





}



